# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    AaddsResourceDetails,
    Application,
    ApplicationGetEndpoint,
    ApplicationGetHttpsEndpoint,
    ApplicationListResult,
    ApplicationProperties,
    AsyncOperationResult,
    Autoscale,
    AutoscaleCapacity,
    AutoscaleConfigurationUpdateParameter,
    AutoscaleRecurrence,
    AutoscaleSchedule,
    AutoscaleTimeAndCapacity,
    AzureMonitorRequest,
    AzureMonitorResponse,
    AzureMonitorSelectedConfigurations,
    AzureMonitorTableConfiguration,
    BillingMeters,
    BillingResources,
    BillingResponseListResult,
    CapabilitiesResult,
    ClientGroupInfo,
    Cluster,
    ClusterConfigurations,
    ClusterCreateParametersExtended,
    ClusterCreateProperties,
    ClusterCreateRequestValidationParameters,
    ClusterCreateValidationResult,
    ClusterDefinition,
    ClusterDiskEncryptionParameters,
    ClusterGetProperties,
    ClusterIdentity,
    ClusterListPersistedScriptActionsResult,
    ClusterListResult,
    ClusterMonitoringRequest,
    ClusterMonitoringResponse,
    ClusterPatchParameters,
    ClusterResizeParameters,
    ComputeIsolationProperties,
    ComputeProfile,
    ConnectivityEndpoint,
    DataDisksGroups,
    Dimension,
    DiskBillingMeters,
    DiskEncryptionProperties,
    EncryptionInTransitProperties,
    ErrorResponse,
    Errors,
    ExcludedServicesConfig,
    ExecuteScriptActionParameters,
    Extension,
    GatewaySettings,
    HardwareProfile,
    HostInfo,
    IPConfiguration,
    KafkaRestProperties,
    LinuxOperatingSystemProfile,
    LocalizedName,
    MetricSpecifications,
    NameAvailabilityCheckRequestParameters,
    NameAvailabilityCheckResult,
    NetworkProperties,
    Operation,
    OperationDisplay,
    OperationListResult,
    OperationProperties,
    OsProfile,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionListResult,
    PrivateLinkConfiguration,
    PrivateLinkResource,
    PrivateLinkResourceListResult,
    PrivateLinkServiceConnectionState,
    ProxyResource,
    QuotaCapability,
    QuotaInfo,
    RegionalQuotaCapability,
    RegionsCapability,
    Resource,
    ResourceAutoGenerated,
    ResourceId,
    Role,
    RuntimeScriptAction,
    RuntimeScriptActionDetail,
    ScriptAction,
    ScriptActionExecutionHistoryList,
    ScriptActionExecutionSummary,
    ScriptActionPersistedGetResponseSpec,
    ScriptActionsList,
    SecurityProfile,
    ServiceSpecification,
    SshProfile,
    SshPublicKey,
    StorageAccount,
    StorageProfile,
    SystemData,
    TrackedResource,
    UpdateClusterIdentityCertificateParameters,
    UpdateGatewaySettingsParameters,
    Usage,
    UsagesListResult,
    UserAssignedIdentity,
    ValidationErrorInfo,
    VersionSpec,
    VersionsCapability,
    VirtualNetworkProfile,
    VmSizeCompatibilityFilterV2,
    VmSizeProperty,
)

from ._hd_insight_management_client_enums import (  # type: ignore
    AsyncOperationState,
    CreatedByType,
    DaysOfWeek,
    DirectoryType,
    FilterMode,
    HDInsightClusterProvisioningState,
    JsonWebKeyEncryptionAlgorithm,
    OSType,
    PrivateEndpointConnectionProvisioningState,
    PrivateIPAllocationMethod,
    PrivateLink,
    PrivateLinkConfigurationProvisioningState,
    PrivateLinkServiceConnectionStatus,
    ResourceIdentityType,
    ResourceProviderConnection,
    RoleName,
    Tier,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AaddsResourceDetails",
    "Application",
    "ApplicationGetEndpoint",
    "ApplicationGetHttpsEndpoint",
    "ApplicationListResult",
    "ApplicationProperties",
    "AsyncOperationResult",
    "Autoscale",
    "AutoscaleCapacity",
    "AutoscaleConfigurationUpdateParameter",
    "AutoscaleRecurrence",
    "AutoscaleSchedule",
    "AutoscaleTimeAndCapacity",
    "AzureMonitorRequest",
    "AzureMonitorResponse",
    "AzureMonitorSelectedConfigurations",
    "AzureMonitorTableConfiguration",
    "BillingMeters",
    "BillingResources",
    "BillingResponseListResult",
    "CapabilitiesResult",
    "ClientGroupInfo",
    "Cluster",
    "ClusterConfigurations",
    "ClusterCreateParametersExtended",
    "ClusterCreateProperties",
    "ClusterCreateRequestValidationParameters",
    "ClusterCreateValidationResult",
    "ClusterDefinition",
    "ClusterDiskEncryptionParameters",
    "ClusterGetProperties",
    "ClusterIdentity",
    "ClusterListPersistedScriptActionsResult",
    "ClusterListResult",
    "ClusterMonitoringRequest",
    "ClusterMonitoringResponse",
    "ClusterPatchParameters",
    "ClusterResizeParameters",
    "ComputeIsolationProperties",
    "ComputeProfile",
    "ConnectivityEndpoint",
    "DataDisksGroups",
    "Dimension",
    "DiskBillingMeters",
    "DiskEncryptionProperties",
    "EncryptionInTransitProperties",
    "ErrorResponse",
    "Errors",
    "ExcludedServicesConfig",
    "ExecuteScriptActionParameters",
    "Extension",
    "GatewaySettings",
    "HardwareProfile",
    "HostInfo",
    "IPConfiguration",
    "KafkaRestProperties",
    "LinuxOperatingSystemProfile",
    "LocalizedName",
    "MetricSpecifications",
    "NameAvailabilityCheckRequestParameters",
    "NameAvailabilityCheckResult",
    "NetworkProperties",
    "Operation",
    "OperationDisplay",
    "OperationListResult",
    "OperationProperties",
    "OsProfile",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionListResult",
    "PrivateLinkConfiguration",
    "PrivateLinkResource",
    "PrivateLinkResourceListResult",
    "PrivateLinkServiceConnectionState",
    "ProxyResource",
    "QuotaCapability",
    "QuotaInfo",
    "RegionalQuotaCapability",
    "RegionsCapability",
    "Resource",
    "ResourceAutoGenerated",
    "ResourceId",
    "Role",
    "RuntimeScriptAction",
    "RuntimeScriptActionDetail",
    "ScriptAction",
    "ScriptActionExecutionHistoryList",
    "ScriptActionExecutionSummary",
    "ScriptActionPersistedGetResponseSpec",
    "ScriptActionsList",
    "SecurityProfile",
    "ServiceSpecification",
    "SshProfile",
    "SshPublicKey",
    "StorageAccount",
    "StorageProfile",
    "SystemData",
    "TrackedResource",
    "UpdateClusterIdentityCertificateParameters",
    "UpdateGatewaySettingsParameters",
    "Usage",
    "UsagesListResult",
    "UserAssignedIdentity",
    "ValidationErrorInfo",
    "VersionSpec",
    "VersionsCapability",
    "VirtualNetworkProfile",
    "VmSizeCompatibilityFilterV2",
    "VmSizeProperty",
    "AsyncOperationState",
    "CreatedByType",
    "DaysOfWeek",
    "DirectoryType",
    "FilterMode",
    "HDInsightClusterProvisioningState",
    "JsonWebKeyEncryptionAlgorithm",
    "OSType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateIPAllocationMethod",
    "PrivateLink",
    "PrivateLinkConfigurationProvisioningState",
    "PrivateLinkServiceConnectionStatus",
    "ResourceIdentityType",
    "ResourceProviderConnection",
    "RoleName",
    "Tier",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
